# Generated by Django 3.2.5 on 2021-10-06 17:43

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0036_auto_20211006_1743'),
    ]

    operations = [
        migrations.AlterField(
            model_name='datatransformerconstants',
            name='id',
            field=models.UUIDField(blank=True, default=uuid.UUID('9a3515db-6b03-4be1-bd5e-949656b55c10'), primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='hardwarestatsdatatransformer',
            name='id',
            field=models.UUIDField(blank=True, default=uuid.UUID('23a04c6b-70d5-4825-be8d-038a6c084c14'), primary_key=True, serialize=False),
        ),
    ]



from django.shortcuts import render

# Create your views here.

from django.http import JsonResponse
from django.core import serializers
from .forms.forms import CalibrationForm
from .models.models import Calibration


def index(request):
    context = {'title': 'HomeHub'}
    return render(request, 'home.html', context)
#
#
# def postFriend(request):
#     # request should be ajax and method should be POST.
#     if request.is_ajax and request.method == "POST":
#         # get the form data
#         form = FriendForm(request.POST)
#         # save the data and after fetch the object in instance
#         if form.is_valid():
#             instance = form.save()
#             # serialize in new friend object in json
#             ser_instance = serializers.serialize('json', [instance, ])
#             # send to client side.
#             return JsonResponse({"instance": ser_instance}, status=200)
#         else:
#             # some form errors occured.
#             return JsonResponse({"error": form.errors}, status=400)
#
#     # some error occured
#     return JsonResponse({"error": ""}, status=400)
#
#
# # chat/views.py
# from django.shortcuts import render
#
#
# def start(request):
#     return render(request, 'start.html', {})
#
#
# def calibrate(request, room_name):
#     return render(request, 'calibrate.html', {
#         'room_name': room_name
#     })



# Generated by Django 3.2.5 on 2021-09-24 20:39

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0018_auto_20210814_1837'),
    ]

    operations = [
        migrations.CreateModel(
            name='PiPicoACAnalogIo',
            fields=[
                ('hardwareio_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='data.hardwareio')),
                ('pin', models.IntegerField()),
            ],
            options={
                'abstract': False,
                'base_manager_name': 'objects',
            },
            bases=('data.hardwareio',),
        ),
        migrations.AlterField(
            model_name='hardwareio',
            name='type',
            field=models.CharField(choices=[('SPI', 'SPI'), ('Serial', 'Serial'), ('PWM', 'PWM'), ('I2C', 'I2C'), ('Device File', 'Device File'), ('MCP Channel', 'MCP Channel'), ('Pi Pico Analog', 'Pi Pico Analog'), ('Pi Pico AC Analog', 'Pi Pico AC Analog'), ('Pi GPIO', 'Pi GPIO')], default='Pi Pico Analog', max_length=25),
        ),
    ]



# Generated by Django 3.2.5 on 2021-10-06 17:43

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0035_auto_20211106_1530'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='datatransformerconstants',
            name='type',
        ),
        migrations.AlterField(
            model_name='datatransformerconstants',
            name='id',
            field=models.UUIDField(blank=True, default=uuid.UUID('66c30765-c9f0-48b9-92a5-b58b8b72f54d'), primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='datatransformerconstants',
            name='index',
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='datatransformerconstants',
            name='value',
            field=models.FloatField(blank=True, default=0.0),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='hardwaredatatransformer',
            name='hardware',
            field=models.ManyToManyField(blank=True, to='data.Channel'),
        ),
        migrations.AlterField(
            model_name='hardwarestatsdatatransformer',
            name='id',
            field=models.UUIDField(blank=True, default=uuid.UUID('2814183a-c039-48ab-8743-846ed7e2bdf2'), primary_key=True, serialize=False),
        ),
    ]



# Generated by Django 3.2.5 on 2021-08-14 18:37

from django.db import migrations

def gen(apps, schema_editor):
    from data.models.models import DataTransformerTypes

    types = ['SUM', 'PRODUCT', 'DIFFERENCE', 'QUOTIENT', 'MAX', 'MIN', 'MEAN']

    for type in types:
        cat = DataTransformerTypes.objects.create()
        cat.type = type
        cat.save()

class Migration(migrations.Migration):

    dependencies = [
        ('data', '0034_auto_20211006_1529'),
    ]

    operations = [
        migrations.RunPython(gen, reverse_code=migrations.RunPython.noop),
    ]


# Generated by Django 3.2.5 on 2021-10-06 17:45

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0037_auto_20211006_1743'),
    ]

    operations = [
        migrations.AlterField(
            model_name='datatransformerconstants',
            name='id',
            field=models.UUIDField(blank=True, default=uuid.UUID('a482c481-6574-4436-991e-926f72a41a30'), primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='hardwarestatsdatatransformer',
            name='id',
            field=models.UUIDField(blank=True, default=uuid.UUID('1444a3d3-ad06-4039-bbcf-00246699b5fb'), primary_key=True, serialize=False),
        ),
    ]



# Generated by Django 3.2.5 on 2021-10-03 01:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0024_auto_20211003_0050'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='hub',
            name='keep_statistics',
        ),
        migrations.AddField(
            model_name='channel',
            name='keep_statistics',
            field=models.BooleanField(blank=True, default=False, null=True),
        ),
    ]



import socket

from .manage.manage import main


def get_my_ip():
    """
    Find my IP address
    :return:
    """
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect(("8.8.8.8", 80))
    ip = s.getsockname()[0]
    s.close()
    return ip


ip = str(get_my_ip())

main('__main__.py','makemigrations', 'data')
main('__main__.py','migrate', 'data')
main('__main__.py','makemigrations')
main('__main__.py','migrate')
main('__main__.py','runserver',ip + ':8000')



# Generated by Django 3.2.5 on 2021-10-06 15:29

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0033_auto_20211005_2134'),
    ]

    operations = [
        migrations.CreateModel(
            name='DataTransformerTypes',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('type', models.CharField(blank=True, default=None, max_length=15, null=True)),
            ],
        ),
        migrations.AlterField(
            model_name='datatransformer',
            name='type',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='data.datatransformertypes'),
        ),
    ]



# Generated by Django 3.2.5 on 2021-10-02 22:32

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0022_auto_20210929_1837'),
    ]

    operations = [
        migrations.CreateModel(
            name='DataTransformer',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('accessory', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='data.accessory')),
                ('data_transformer_child', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='data_child', to='data.datatransformer')),
                ('data_transformer_parent', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='data_parent', to='data.datatransformer')),
            ],
        ),
        migrations.CreateModel(
            name='HardwareStats',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('type', models.CharField(max_length=255, null=True)),
                ('value', models.FloatField(null=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('hardware', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='data.hardware')),
            ],
        ),
        migrations.AddField(
            model_name='hub',
            name='keep_statistics',
            field=models.BooleanField(blank=True, default=False, null=True),
        ),
        migrations.CreateModel(
            name='HardwareStatsDataTransformer',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('index', models.IntegerField()),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('data_transformer', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='data.datatransformer')),
                ('hardware_stat', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='data.hardwarestats')),
            ],
        ),
        migrations.CreateModel(
            name='HardwareDataTransformer',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('index', models.IntegerField()),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('data_transformer', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='data.datatransformer')),
                ('hardware', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='data.hardware')),
            ],
        ),
        migrations.CreateModel(
            name='DataTransformerConstants',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('type', models.CharField(max_length=255, null=True)),
                ('index', models.IntegerField()),
                ('value', models.FloatField(null=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('data_transformer', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='data.datatransformer')),
            ],
        ),
        migrations.AddIndex(
            model_name='hardwarestatsdatatransformer',
            index=models.Index(fields=['id', 'data_transformer'], name='data_hardwa_id_b714f1_idx'),
        ),
        migrations.AddIndex(
            model_name='hardwarestats',
            index=models.Index(fields=['id', 'hardware'], name='data_hardwa_id_5b03a1_idx'),
        ),
        migrations.AddIndex(
            model_name='hardwaredatatransformer',
            index=models.Index(fields=['id', 'data_transformer', 'hardware'], name='data_hardwa_id_c5fb61_idx'),
        ),
        migrations.AddIndex(
            model_name='datatransformerconstants',
            index=models.Index(fields=['id', 'data_transformer'], name='data_datatr_id_94efc8_idx'),
        ),
        migrations.AddIndex(
            model_name='datatransformer',
            index=models.Index(fields=['id', 'accessory'], name='data_datatr_id_9bcd89_idx'),
        ),
    ]



import uuid

from django import forms

from django.forms import ChoiceField

from data.models.models import Calibration, Hardware, Accessory, Channel, HardwareConfig, \
    CalibrationConstants, DHT22, MCP3008, \
    ModProbe, PiPico, VEML7700, HardwareChannelTypes, Hub, Category, AccessoryType, SPIIo, SerialIo, PwmIo, I2cIo, \
    DeviceFileIo, MCPAnalogIo, PiGpio, PiPicoAnalogIo, PiPicoACAnalogIo, HardwareIO, PMSA0031


class HubForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(HubForm, self).__init__(*args, **kwargs)
        ## add a "form-control" class to each form input
        ## for enabling bootstrap
        categories = list(
            Category.objects.all().values_list('enum', 'name'))
        # categories = [('----', '-----')]
        # for chan_type in channel_types_temp:
        #     channel_types.append((chan_type['channel_type'], chan_type['channel_type']))

        for name in self.fields.keys():
            self.fields[name].widget.attrs.update({
                'class': 'form-control',
            })

        self.fields['category'] = ChoiceField(choices=tuple(categories))

    class Meta:
        model = Hub
        fields = ("__all__")


class CalibrationConstantForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(CalibrationConstantForm, self).__init__(*args, **kwargs)
        ## add a "form-control" class to each form input
        ## for enabling bootstrap

        for name in self.fields.keys():
            if name == 'calibration':
                self.fields[name].widget = forms.HiddenInput()
            self.fields[name].widget.attrs.update({
                'class': 'form-control',
            })

    class Meta:
        model = CalibrationConstants
        fields = ("__all__")


class CalibrationForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(CalibrationForm, self).__init__(*args, **kwargs)
        ## add a "form-control" class to each form input
        ## for enabling bootstrap
        for name in self.fields.keys():
            if name == 'accessory':
                self.fields[name].widget = forms.HiddenInput()
            else:
                self.fields[name].widget.attrs.update({
                    'class': 'form-control',
                })

    class Meta:
        model = Calibration
        fields = ("__all__")


class HardwareForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(HardwareForm, self).__init__(*args, **kwargs)
        ## add a "form-control" class to each form input
        ## for enabling bootstrap
        print('asdf')
        for name in self.fields.keys():
            if name == 'type':
                self.fields[name].widget = forms.HiddenInput()
            print(name)
            self.fields[name].widget.attrs.update({
                'class': 'form-control',
            })

    class Meta:
        model = Hardware
        fields = ("__all__")


class HardwareDHT22Form(HardwareForm):
    def __init__(self, *args, **kwargs):
        super(HardwareDHT22Form, self).__init__(*args, **kwargs)

    class Meta:
        model = DHT22
        fields = ("__all__")


class HardwareMCP3008Form(HardwareForm):
    def __init__(self, *args, **kwargs):
        super(HardwareMCP3008Form, self).__init__(*args, **kwargs)

    class Meta:
        model = MCP3008
        fields = ("__all__")


class HardwareModProbeForm(HardwareForm):
    def __init__(self, *args, **kwargs):
        super(HardwareModProbeForm, self).__init__(*args, **kwargs)

    class Meta:
        model = ModProbe
        fields = ("__all__")


class HardwarePiPicoForm(HardwareForm):
    def __init__(self, *args, **kwargs):
        super(HardwarePiPicoForm, self).__init__(*args, **kwargs)

    class Meta:
        model = PiPico
        fields = ("__all__")


class HardwareVEML7700Form(HardwareForm):
    def __init__(self, *args, **kwargs):
        super(HardwareVEML7700Form, self).__init__(*args, **kwargs)

    class Meta:
        model = VEML7700
        fields = ("__all__")


class HardwarePMSA0031Form(HardwareForm):
    def __init__(self, *args, **kwargs):
        super(HardwarePMSA0031Form, self).__init__(*args, **kwargs)

    class Meta:
        model = PMSA0031
        fields = ("__all__")


class AccessoryForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(AccessoryForm, self).__init__(*args, **kwargs)
        ## add a "form-control" class to each form input
        ## for enabling bootstrap
        for name in self.fields.keys():
            # if name == 'channels':
            #     self.fields[name]= forms.ModelChoiceField(queryset=Channel.objects.all().values(('id')), empty_label="(Nothing)")
            # else:
            self.fields[name].widget.attrs.update({
                'class': 'form-control',
            })

        categories = list(
            Category.objects.all().values_list('enum', 'name'))
        self.fields['category'] = ChoiceField(choices=tuple(categories))

        types = list(
            AccessoryType.objects.all().values_list('type', 'type'))
        self.fields['type'] = ChoiceField(choices=tuple(types))

    class Meta:
        model = Accessory
        fields = ("__all__")


class ChannelForm(forms.ModelForm):

    def __init__(self, hardware_type=None, *args, **kwargs):
        super(ChannelForm, self).__init__(*args, **kwargs)
        if hardware_type is not None:
            channel_types_temp = list(
                HardwareChannelTypes.objects.filter(hardware_type=hardware_type).values('channel_type'))
            channel_types = [('----', '-----')]
            for chan_type in channel_types_temp:
                channel_types.append((chan_type['channel_type'], chan_type['channel_type']))
            self.fields['type'] = ChoiceField(choices=tuple(channel_types))

        # self.fields['type'] = ChoiceField(queryset=HardwareChannelTypes.objects.filter(hardware_type=hardware_type).values_list('channel_type'))
        # self.fields['type'].queryset = HardwareChannelTypes.objects.filter(hardware_type=hardware_type)

        ## add a "form-control" class to each form input
        ## for enabling bootstrap
        for name in self.fields.keys():
            if name == 'hardware':
                self.fields[name].widget = forms.HiddenInput()
            self.fields[name].widget.attrs.update({
                'class': 'form-control',
            })

    class Meta:
        model = Channel
        fields = ("__all__")


class HardwareConfigForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(HardwareConfigForm, self).__init__(*args, **kwargs)
        ## add a "form-control" class to each form input
        ## for enabling bootstrap
        for name in self.fields.keys():
            self.fields[name].widget.attrs.update({
                'class': 'form-control',
            })

    class Meta:
        model = HardwareConfig
        fields = ("__all__")


class HardwareIoForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(HardwareIoForm, self).__init__(*args, **kwargs)
        # self.fields['type'] = ChoiceField(choices=tuple(channel_types))

        ## add a "form-control" class to each form input
        ## for enabling bootstrap
        # print('asdf')
        # for name in self.fields.keys():
        #     if name in ('parent_hardware', 'child_hardware', 'id', 'child_channel', 'hub'):
        #         self.fields[name].widget = forms.HiddenInput()
        #     print(name)
        #     self.fields[name].widget.attrs.update({
        #         'class': 'form-control',
        #     })

    class Meta:
        model = HardwareIO
        fields = ("__all__")


class SPIIoForm(HardwareIoForm):

    def __init__(self, *args, **kwargs):
        super(SPIIoForm, self).__init__(*args, **kwargs)
        ## add a "form-control" class to each form input
        ## for enabling bootstrap
        print('asdf')
        for name in self.fields.keys():
            if name in ('parent_hardware', 'child_hardware', 'id', 'child_channel', 'hub'):
                self.fields[name].widget = forms.HiddenInput()
            print(name)
            self.fields[name].widget.attrs.update({
                'class': 'form-control',
            })

    class Meta:
        model = SPIIo
        fields = ("__all__")


class SerialIoForm(HardwareIoForm):

    def __init__(self, *args, **kwargs):
        super(SerialIoForm, self).__init__(*args, **kwargs)
        ## add a "form-control" class to each form input
        ## for enabling bootstrap
        print('asdf')
        for name in self.fields.keys():
            if name in ('parent_hardware', 'child_hardware', 'id', 'child_channel', 'hub'):
                self.fields[name].widget = forms.HiddenInput()
            print(name)
            self.fields[name].widget.attrs.update({
                'class': 'form-control',
            })

    class Meta:
        model = SerialIo
        fields = ("__all__")


class PwmIoForm(HardwareIoForm):
    def __init__(self, *args, **kwargs):
        super(PwmIoForm, self).__init__(*args, **kwargs)
        ## add a "form-control" class to each form input
        ## for enabling bootstrap
        print('asdf')
        for name in self.fields.keys():
            if name in ('parent_hardware', 'child_hardware', 'id', 'child_channel', 'hub'):
                self.fields[name].widget = forms.HiddenInput()
            print(name)
            self.fields[name].widget.attrs.update({
                'class': 'form-control',
            })

    class Meta:
        model = PwmIo
        fields = ("__all__")


class I2cIoForm(HardwareIoForm):

    def __init__(self, *args, **kwargs):
        super(I2cIoForm, self).__init__(*args, **kwargs)
        ## add a "form-control" class to each form input
        ## for enabling bootstrap
        print('asdf')
        for name in self.fields.keys():
            if name in ('parent_hardware', 'child_hardware', 'id', 'child_channel', 'hub'):
                self.fields[name].widget = forms.HiddenInput()
            print(name)
            self.fields[name].widget.attrs.update({
                'class': 'form-control',
            })

    class Meta:
        model = I2cIo
        fields = ("__all__")


class DeviceFileIoForm(HardwareIoForm):

    def __init__(self, *args, **kwargs):
        super(DeviceFileIoForm, self).__init__(*args, **kwargs)
        ## add a "form-control" class to each form input
        ## for enabling bootstrap
        print('asdf')
        for name in self.fields.keys():
            if name in ('parent_hardware', 'child_hardware', 'id', 'child_channel', 'hub'):
                self.fields[name].widget = forms.HiddenInput()
            print(name)
            self.fields[name].widget.attrs.update({
                'class': 'form-control',
            })

    class Meta:
        model = DeviceFileIo
        fields = ("__all__")


class MCPAnalogIoForm(HardwareIoForm):

    def __init__(self, *args, **kwargs):
        super(MCPAnalogIoForm, self).__init__(*args, **kwargs)
        ## add a "form-control" class to each form input
        ## for enabling bootstrap
        print('asdf')
        for name in self.fields.keys():
            if name in ('parent_hardware', 'child_hardware', 'id', 'child_channel', 'hub'):
                self.fields[name].widget = forms.HiddenInput()
            print(name)
            self.fields[name].widget.attrs.update({
                'class': 'form-control',
            })

    class Meta:
        model = MCPAnalogIo
        fields = ("__all__")


class PiPicoAnalogIoForm(HardwareIoForm):

    def __init__(self, *args, **kwargs):
        super(PiPicoAnalogIoForm, self).__init__(*args, **kwargs)
        ## add a "form-control" class to each form input
        ## for enabling bootstrap
        print('asdf')
        for name in self.fields.keys():
            if name in ('parent_hardware', 'child_hardware', 'id', 'child_channel', 'hub'):
                self.fields[name].widget = forms.HiddenInput()
            print(name)
            self.fields[name].widget.attrs.update({
                'class': 'form-control',
            })

    class Meta:
        model = PiPicoAnalogIo
        fields = ("__all__")


class PiPicoACAnalogIoForm(HardwareIoForm):

    def __init__(self, *args, **kwargs):
        super(PiPicoACAnalogIoForm, self).__init__(*args, **kwargs)
        ## add a "form-control" class to each form input
        ## for enabling bootstrap
        print('asdf')
        for name in self.fields.keys():
            if name in ('parent_hardware', 'child_hardware', 'id', 'child_channel', 'hub'):
                self.fields[name].widget = forms.HiddenInput()
            print(name)
            self.fields[name].widget.attrs.update({
                'class': 'form-control',
            })

    class Meta:
        model = PiPicoACAnalogIo
        fields = ("__all__")


class PiGpioForm(HardwareIoForm):

    def __init__(self, *args, **kwargs):
        super(PiGpioForm, self).__init__(*args, **kwargs)
        ## add a "form-control" class to each form input
        ## for enabling bootstrap
        print('asdf')
        for name in self.fields.keys():
            if name in ('parent_hardware', 'child_hardware', 'id', 'child_channel', 'hub'):
                self.fields[name].widget = forms.HiddenInput()
            print(name)
            self.fields[name].widget.attrs.update({
                'class': 'form-control',
            })

    class Meta:
        model = PiGpio
        fields = ("__all__")


DHT22 = 'DHT22'
MCP3008 = 'MCP3008'
ModProbe = 'ModProbe'
PiPico = 'PiPico'
VEML7700 = 'VEML7700'
PMSA0031 = 'PMSA0031'
CHOICES = (('DHT22', 'DHT22'), ('MCP3008', 'MCP3008'), ('ModProbe', 'ModProbe'), ('PiPico', 'PiPico'),
           ('VEML7700', 'VEML7700'), ('PMSA0031', 'PMSA0031'))


class HardwareTypeForm(forms.Form):
    # def __init__(self, *args, **kwargs):
    #     super(HardwareTypeForm, self).__init__(*args, **kwargs)

    type = forms.ChoiceField(choices=CHOICES)


SPI = 'SPI'
Serial = 'Serial'
PWM = 'PWM'
I2C = 'I2C'
DeviceFile = 'Device File'
MCPChannel = 'MCP Channel'
PiPicoAnalog = 'Pi Pico Analog'
PiPicoACAnalog = 'Pi Pico AC Analog'
PiGPIO = 'Pi GPIO'

hardware_io_choices = (
    (SPI, SPI), (Serial, Serial), (PWM, PWM), (I2C, I2C), (DeviceFile, DeviceFile), (MCPChannel, MCPChannel),
    (PiPicoAnalog, PiPicoAnalog), (PiPicoACAnalog, PiPicoACAnalog), (PiGPIO, PiGPIO))


class HardwareIOTypeForm(forms.Form):
    type = forms.ChoiceField(choices=hardware_io_choices)
    parent_hardware = forms.CharField()
    child_hardware = forms.CharField()
    child_channel = forms.CharField()
    hub = forms.CharField()
    parent_hardware.widget = forms.HiddenInput()
    child_hardware.widget = forms.HiddenInput()
    child_channel.widget = forms.HiddenInput()
    hub.widget = forms.HiddenInput()

    # def __init__(self, *args, **kwargs):
    #     super(HardwareTypeForm, self).__init__(*args, **kwargs)
    def __init__(self, *args, **kwargs):
        super(HardwareIOTypeForm, self).__init__(*args, **kwargs)

    class Meta:
        fields = ['type', 'parent_hardware', 'child_hardware', 'child_channel', 'hub']


from django.forms.models import inlineformset_factory, BaseModelFormSet, BaseInlineFormSet, BaseModelForm
from data.models.models import HardwareStats, DataTransformer, DataTransformerConstants, HardwareDataTransformer, \
    HardwareStatsDataTransformer, DataTransformerTypes


# DataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformer, fk_name='data_transformer_parent',
#                                                      extra=1,exclude =['updated_at','created_at'],formset=DataTransformerForm)

class DataConstantsTransformerForm(forms.ModelForm):
    id = forms.CharField(required=False,
                         widget=forms.HiddenInput(attrs={'class': 'form-control'}))

    index = forms.IntegerField(required=False,
                               widget=forms.HiddenInput(attrs={'class': 'form-control'}),
                               initial=0)
    value = forms.CharField(required=False,
                            widget=forms.TextInput(attrs={'class': 'form-control'}))
    data_transformer = forms.ModelChoiceField(required=False, queryset=DataTransformer.objects.all(),
                                              widget=forms.HiddenInput(attrs={'class': 'form-control'}))

    class Meta:
        model = DataTransformerConstants
        fields = ('__all__')


class HardwareTransformerForm(forms.ModelForm):
    id = forms.CharField(required=False,
                         widget=forms.HiddenInput(attrs={'class': 'form-control'}))
    index = forms.IntegerField(required=False,
                               widget=forms.HiddenInput(attrs={'class': 'form-control'}),
                               initial=0)
    data_transformer = forms.ModelChoiceField(required=False, queryset=DataTransformer.objects.all(),
                                              widget=forms.HiddenInput(attrs={'class': 'form-control'}))

    class Meta:
        model = HardwareDataTransformer
        fields = ('__all__')


class HardwareStatsTransformerForm(forms.ModelForm):
    id = forms.CharField(required=False,
                         widget=forms.HiddenInput(attrs={'class': 'form-control'}))
    index = forms.IntegerField(required=False,
                               widget=forms.HiddenInput(attrs={'class': 'form-control'}),
                               initial=0)
    data_transformer = forms.ModelChoiceField(required=False, queryset=DataTransformer.objects.all(),
                                              widget=forms.HiddenInput(attrs={'class': 'form-control'}))

    class Meta:
        model = HardwareStatsDataTransformer
        fields = ('__all__')


class BaseInlineFormSetWithAddMoreButton(BaseInlineFormSet):
    def as_p(self):
        button_id = str(self.prefix) + 'add_more'
        empty_form_id = str(self.prefix) + 'empty_form'
        empty_form_as_p = str(self.empty_form.as_p())
        div_name = str(self.prefix) + 'div'

        forms_as_p = BaseInlineFormSet.as_p(self)
        p = f"""<div class="row" id="{div_name}">
        {forms_as_p}
        </div>
        <input type="button" value="Add More" id="{button_id}">
                    <div id="{empty_form_id}" style="display:none">
                    {empty_form_as_p}
                    </div>
                    <script src="https://cdnjs.cloudflare.com/ajax/libs/jquery/3.3.1/jquery.min.js"></script>
<script src="https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.12.9/umd/popper.min.js"></script>
<script src="https://maxcdn.bootstrapcdn.com/bootstrap/4.0.0/js/bootstrap.min.js"></script>
        <script>
    $('#{button_id}').click(function() {{
        var form_idx = $('#id_{self.prefix}-TOTAL_FORMS').val();
        var empty_form = document.getElementById('{empty_form_id}').innerHTML.replace(/__prefix__/g, form_idx);        
        
        var d1 = document.getElementById('{div_name}');
        d1.insertAdjacentHTML('afterend', empty_form);
        $('#id_{self.prefix}-TOTAL_FORMS').val(parseInt(form_idx) + 1);
    }});
</script>"""
        print(p)
        return p


DataConstantsTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformerConstants, formset=BaseInlineFormSetWithAddMoreButton, extra=0,
                                                              form=DataConstantsTransformerForm, can_delete_extra=True)
# HardwareTransformerInlineFormset = inlineformset_factory(DataTransformer, HardwareDataTransformer, extra=1,
#                                                          form=HardwareTransformerForm, can_delete_extra=True)
HardwareStatsTransformerInlineFormset = inlineformset_factory(DataTransformer, HardwareStatsDataTransformer, extra=0, formset=BaseInlineFormSetWithAddMoreButton,
                                                              form=HardwareStatsTransformerForm, can_delete_extra=True)

from django.utils.safestring import mark_safe


class DataTransform(forms.ModelForm):
    id = forms.CharField(required=False,
                         widget=forms.HiddenInput(attrs={'class': 'form-control'}),
                         initial=uuid.uuid4())
    accessory = forms.ModelChoiceField(required=False, queryset=Accessory.objects.all(),
                                       widget=forms.HiddenInput(attrs={'class': 'form-control'}))

    parent = forms.ModelChoiceField(required=False, queryset=DataTransformer.objects.all(),
                                    widget=forms.HiddenInput(attrs={'class': 'form-control'}))

    type = forms.ModelChoiceField(required=False, queryset=DataTransformerTypes.objects.all())

    def __init__(self, *args, **kwargs):
        print('Init Data Transform')

        super(DataTransform, self).__init__(*args, **kwargs)
        if self.fields['parent'] is not None and self.fields['parent'] != '':
            self.fields['accessory'].initial = str(DataTransformer.objects.get(self.fields['parent'].initial).accessory.pk)

        try:
            if self.instance.get_children().count() >= 0:
                print('Adding to child form node: ' + str(self.instance.get_children().count() + 1))

                self.nested_data_transformer_form = dataTransformerInlineFormset(
                    instance=self.instance,
                    queryset=self.instance.get_children(),
                    data=self.data if self.is_bound else None,
                    files=self.files if self.is_bound else None,
                    initial=[{'parent': str(self.instance.pk), 'accessory': str(self.instance.accessory.id),
                              'id': str(uuid.uuid4())}],
                    prefix=str(self.instance.id) + 'dataTransformerInlineFormset'
                )
                self.nested_data_transformer_form.extra = self.instance.get_children().count()


        except Exception as e:
            print(str(e))
            # self.nested_data_transformer_form = dataTransformerInlineFormset(
            #     instance=self.instance,
            #     data=self.data if self.is_bound else None,
            #     files=self.files if self.is_bound else None,
            #     prefix=str(self.instance.id) + 'dataTransformerInlineFormset'
            # )
            # self.nested_data_transformer_form.extra = 0

        self.nested_constants_form = DataConstantsTransformerInlineFormset(
            instance=self.instance,
            queryset=self.instance.datatransformerconstants_set,
            data=self.data if self.is_bound else None,
            files=self.files if self.is_bound else None,
            prefix=str(self.instance.id) + 'DataConstantsTransformerInlineFormset',
            initial=[{'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}]

        )
        self.nested_constants_form.extra = self.instance.datatransformerconstants_set.count()

        hardware_outputs = self.instance.hardwaredatatransformer_set.first()
        self.nested_hardware_outputs_form = HardwareTransformerForm(
            instance=hardware_outputs,
            data=self.data if self.is_bound else None,
            files=self.files if self.is_bound else None,
            prefix=str(self.instance.id) + 'HardwareTransformerForm',
            initial={'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}
        )

        self.nested_hardware_stats_form = HardwareStatsTransformerInlineFormset(
            instance=self.instance,
            queryset=self.instance.hardwarestatsdatatransformer_set,
            data=self.data if self.is_bound else None,
            files=self.files if self.is_bound else None,
            prefix=str(self.instance.id) + 'HardwareStatsTransformerInlineFormset',
            initial=[{'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}]

        )
        self.nested_hardware_stats_form.extra = self.instance.hardwarestatsdatatransformer_set.count()

    def as_p(self):
        print('wtf')
        print('instance: ' + str(self.instance.id))
        forms = '<div class="row"><div class="col-6">'

        forms = super(DataTransform, self).as_p()
        forms = forms + '</div><div class="col-6">'

        if hasattr(self, 'nested_constants_form') and self.nested_constants_form is not None:
            forms = forms + '\n  -' + self.nested_constants_form.as_p()
        if hasattr(self, 'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None:
            forms = forms + '\n  -' + self.nested_hardware_outputs_form.as_p()
        if hasattr(self, 'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None:
            forms = forms + '\n  -' + self.nested_hardware_stats_form.as_p()
        if hasattr(self, 'nested_data_transformer_form') and self.nested_data_transformer_form is not None:
            print('netdatatransform with number elements: ' + str(len(self.nested_data_transformer_form)))
            forms = forms + '\n       -' + self.nested_data_transformer_form.as_p()
        forms = forms + '</div></div>'

        return mark_safe(forms)

    def save(self, commit=True):
        print('node')
        print('save: ' + str(self.instance.id))
        root_node = super(DataTransform, self).save(commit)
        if hasattr(self,
                   'nested_constants_form') and self.nested_constants_form is not None:
            self.nested_constants_form.instance.refresh_from_db()
            if self.nested_constants_form.is_valid():
                self.nested_constants_form.save(commit)
            else:
                print(self.nested_constants_form.errors)
        if hasattr(self,
                   'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None:

            if self.nested_hardware_outputs_form.is_valid():
                self.nested_hardware_outputs_form.save(commit)
            else:
                print(self.nested_hardware_outputs_form.errors)
        if hasattr(self,
                   'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None:
            self.nested_hardware_stats_form.instance.refresh_from_db()

            if self.nested_hardware_stats_form.is_valid():
                self.nested_hardware_stats_form.save(commit)
            else:
                print(self.nested_hardware_stats_form.errors)
        if hasattr(self,
                   'nested_data_transformer_form') and self.nested_data_transformer_form is not None:
            self.nested_data_transformer_form.instance.refresh_from_db()

            if self.nested_data_transformer_form.is_valid():
                self.nested_data_transformer_form.save(commit)
            else:
                print(self.nested_data_transformer_form.errors)
        root_node.refresh_from_db()
        return root_node

    class Meta:
        model = DataTransformer
        fields = ['id', 'accessory', 'parent', 'type']


# DataTransformerFormset = form_factory(DataTransformer,formset=DataTransformerForm)
dataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformer,
                                                     fk_name='parent',
                                                     can_delete_extra=True,
                                                     form=DataTransform,
                                                     formset=BaseInlineFormSetWithAddMoreButton,
                                                     extra=0
                                                     )


class DataTransformRoot(forms.ModelForm):
    id = forms.CharField(required=False,
                         widget=forms.HiddenInput(attrs={'class': 'form-control'}),initial=uuid.uuid4())
    accessory = forms.ModelChoiceField(required=True, queryset=Accessory.objects.all(),
                                       widget=forms.HiddenInput(attrs={'class': 'form-control'}))
    type = forms.ModelChoiceField(required=True, queryset=DataTransformerTypes.objects.all())

    # def __init__(self, *args, **kwargs):
    #     super(HardwareTypeForm, self).__init__(*args, **kwargs)
    def __init__(self, *args, **kwargs):
        super(DataTransformRoot, self).__init__(*args, **kwargs)
        self.fields['accessory'].initial = str(self.instance.accessory.pk)

        # types = list(
        #     DataTransformerTypes.objects.all().values_list('type', 'type'))
        # self.fields['type'] = ChoiceField(choices=tuple(types))
        try:
            if hasattr(self.instance, 'get_children'):
                print('Adding to root form node: ' + str(self.instance.get_children().count() + 1))

                self.nested_data_transformer_form = dataTransformerInlineFormset(
                    instance=self.instance,
                    queryset=self.instance.get_children(),
                    data=self.data if self.is_bound else None,
                    files=self.files if self.is_bound else None,
                    initial=[{'parent': str(self.instance.pk), 'accessory': str(self.instance.accessory.id),
                              'id': str(uuid.uuid4())}],
                    prefix=str(self.instance.id) + 'dataTransformerInlineFormset',
                )
                self.nested_data_transformer_form.extra = self.instance.get_children().count()

            else:

                self.nested_data_transformer_form = dataTransformerInlineFormset(
                    instance=self.instance,
                    data=self.data if self.is_bound else None,
                    files=self.files if self.is_bound else None,
                    prefix=str(self.instance.id) + 'dataTransformerInlineFormset',
                    initial=[{'parent': str(self.instance.pk), 'accessory': str(self.instance.accessory.id),
                              'id': str(uuid.uuid4())}],
                )
                self.nested_data_transformer_form.extra = 0
        except Exception as e:
            print(str(e))

        self.nested_constants_form = DataConstantsTransformerInlineFormset(
            instance=self.instance,
            queryset=self.instance.datatransformerconstants_set,
            data=self.data if self.is_bound else None,
            files=self.files if self.is_bound else None,
            prefix=str(self.instance.id) + 'DataConstantsTransformerInlineFormset',
            initial=[{'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}]

        )
        self.nested_constants_form.extra = self.instance.datatransformerconstants_set.count()

        hardware_outputs = self.instance.hardwaredatatransformer_set.first()
        self.nested_hardware_outputs_form = HardwareTransformerForm(
            instance=hardware_outputs,
            data=self.data if self.is_bound else None,
            files=self.files if self.is_bound else None,
            prefix=str(self.instance.id) + 'HardwareTransformerForm',
            initial={'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}
        )

        self.nested_hardware_stats_form = HardwareStatsTransformerInlineFormset(
            instance=self.instance,
            queryset=self.instance.hardwarestatsdatatransformer_set,
            data=self.data if self.is_bound else None,
            files=self.files if self.is_bound else None,
            prefix=str(self.instance.id) + 'HardwareStatsTransformerInlineFormset',
            initial=[{'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}]

        )
        self.nested_hardware_stats_form.extra = self.instance.hardwarestatsdatatransformer_set.count()

    def as_p(self):
        print('root')
        print('instance: ' + str(self.instance.id))
        forms = '<div class="row"><div class="col-6">'

        forms = forms + super(DataTransformRoot, self).as_p()
        forms = forms + '</div><div class="col-6">'
        if hasattr(self, 'nested_constants_form') and self.nested_constants_form is not None:
            forms = forms + '\n  -' + self.nested_constants_form.as_p()
        if hasattr(self, 'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None:
            forms = forms + '\n  -' + self.nested_hardware_outputs_form.as_p()
        if hasattr(self, 'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None:
            forms = forms + '\n  -' + self.nested_hardware_stats_form.as_p()
        if hasattr(self, 'nested_data_transformer_form') and self.nested_data_transformer_form is not None:
            print('netdatatransform with number elements: ' + str(len(self.nested_data_transformer_form)))
            forms = forms + '\n       -' + self.nested_data_transformer_form.as_p()
        forms = forms + '</div></div>'
        return mark_safe(forms)

    def save(self, commit=True):
        print('root')
        print('save: ' + str(self.instance.id))
        root_node = super(DataTransformRoot, self).save(commit)
        if hasattr(self,
                   'nested_constants_form') and self.nested_constants_form is not None:
            self.nested_constants_form.instance.refresh_from_db()
            if self.nested_constants_form.is_valid():
                self.nested_constants_form.save(commit)
            else:
                print(self.nested_constants_form.errors)
        if hasattr(self,
                   'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None:

            if self.nested_hardware_outputs_form.is_valid():
                self.nested_hardware_outputs_form.save(commit)
            else:
                print(self.nested_hardware_outputs_form.errors)
        if hasattr(self,
                   'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None:
            self.nested_hardware_stats_form.instance.refresh_from_db()

            if self.nested_hardware_stats_form.is_valid():
                self.nested_hardware_stats_form.save(commit)
            else:
                print(self.nested_hardware_stats_form.errors)
        if hasattr(self,
                   'nested_data_transformer_form') and self.nested_data_transformer_form is not None:
            self.nested_data_transformer_form.instance.refresh_from_db()

            if self.nested_data_transformer_form.is_valid():
                self.nested_data_transformer_form.save(commit)
            else:
                print(self.nested_data_transformer_form.errors)
        root_node.refresh_from_db()
        return root_node.get_root()

    class Meta:
        model = DataTransformer
        fields = ['id', 'accessory', 'type']
# DataTransformerFormset = form_factory(DataTransformer,formset=DataTransformerForm)



# Generated by Django 3.2.4 on 2021-07-12 23:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0011_auto_20210712_2041'),
    ]

    operations = [
        migrations.AddField(
            model_name='hardwareio',
            name='type',
            field=models.CharField(choices=[('SPI', 'SPI'), ('Serial', 'Serial'), ('PWM', 'PWM'), ('I2C', 'I2C'), ('Device File', 'Device File'), ('MCP Channel', 'MCP Channel'), ('Pi Pico Analog', 'Pi Pico Analog'), ('Pi GPIO', 'Pi GPIO')], default='Pi Pico Analog', max_length=25),
        ),
    ]



# Generated by Django 3.2.5 on 2021-08-14 18:37

from django.db import migrations


def gen(apps, schema_editor):
    from data.models.models import HardwareChannelTypes

    hardware_channel_types = [('DHT22', 'DHT22Humidity'),
                              ('DHT22', 'DHT22Temp'),
                              ('MCP3008', 'MCP3008Analog'),
                              ('ModProbe', 'ModProbeTemp'),
                              ('VEML7700', 'VEML7700Light'),
                              ('VEML7700', 'VEML7700Lux')
                              ]

    for (hard, channel) in hardware_channel_types:
        cat = HardwareChannelTypes.objects.create()
        cat.hardware_type = hard
        cat.channel_type = channel
        cat.save()


class Migration(migrations.Migration):
    dependencies = [
        ('data', '0020_auto_20210924_1837'),
    ]

    operations = [
        migrations.RunPython(gen, reverse_code=migrations.RunPython.noop),
    ]



# Generated by Django 3.2.5 on 2021-10-03 03:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0025_auto_20211003_0103'),
    ]

    operations = [
        migrations.AddField(
            model_name='datatransformer',
            name='type',
            field=models.CharField(blank=True, default=None, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='hardwaredatatransformer',
            name='index',
            field=models.IntegerField(blank=True, default=None, null=True),
        ),
    ]
